@@ -13,17 +13,16 @@
 #include <debugger/framestack/framestackmodel.h>
 #include <interfaces/icore.h>
 #include <interfaces/idocumentcontroller.h>
-#include <util/environmentprofilelist.h>
 
 #include "debugsession.h"
+#include "breakpointcontroller.h"
 #include "pdbframestackmodel.h"
+#include "startupinfo.h"
 #include "variablecontroller.h"
 #include "variable.h"
-#include "breakpointcontroller.h"
 
 #include <QDebug>
 #include <QRegularExpression>
-#include <QStandardPaths>
 
 #include "debuggerdebug.h"
 
@@ -44,19 +43,15 @@ using namespace KDevelop;
 
 namespace Python {
 
-DebugSession::DebugSession(QStringList program, const QUrl &workingDirectory,
-    const QString& envProfileName) :
-    IDebugSession()
+DebugSession::DebugSession()
+    : IDebugSession()
     , m_breakpointController(nullptr)
     , m_variableController(nullptr)
     , m_frameStackModel(nullptr)
-    , m_workingDirectory(workingDirectory)
-    , m_envProfileName(envProfileName)
     , m_nextNotifyMethod(nullptr)
     , m_inDebuggerData(0)
 {
     qCDebug(KDEV_PYTHON_DEBUGGER) << "creating debug session";
-    m_program = program;
     m_breakpointController = new Python::BreakpointController(this);
     m_variableController = new VariableController(this);
     m_frameStackModel = new PdbFrameStackModel(this);
@@ -77,33 +72,27 @@ IFrameStackModel* DebugSession::frameStackModel() const
     return m_frameStackModel;
 }
 
-void DebugSession::start()
+void DebugSession::start(const StartupInfo& info)
 {
     setState(StartingState);
     m_debuggerProcess = new KProcess(this);
-    m_debuggerProcess->setProgram(m_program);
+    *m_debuggerProcess << info.interpreter << QStringLiteral("-u") << info.debuggerPath << info.scriptPath << info.args;
     m_debuggerProcess->setOutputChannelMode(KProcess::SeparateChannels);
     m_debuggerProcess->blockSignals(true);
-    m_debuggerProcess->setWorkingDirectory(m_workingDirectory.path());
-
-    const KDevelop::EnvironmentProfileList environmentProfiles(KSharedConfig::openConfig());
-    const auto environment = environmentProfiles.variables(m_envProfileName);
-
-    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
-    for(auto i = environment.cbegin(); i != environment.cend(); i++ )
-    {
-        env.insert(i.key(), i.value());
-    }
-    m_debuggerProcess->setProcessEnvironment(env);
+    m_debuggerProcess->setWorkingDirectory(info.workingDirectory.path());
+    m_debuggerProcess->setProcessEnvironment(info.environment);
 
     connect(m_debuggerProcess, &QProcess::readyReadStandardOutput, this, &DebugSession::dataAvailable);
     connect(m_debuggerProcess, SIGNAL(finished(int)), this, SLOT(debuggerQuit(int)));
     connect(this, &DebugSession::debuggerReady, this, &DebugSession::checkCommandQueue);
     connect(this, &DebugSession::commandAdded, this, &DebugSession::checkCommandQueue);
+
+    qCDebug(KDEV_PYTHON_DEBUGGER) << "starting program:" << m_debuggerProcess->program();
+
     m_debuggerProcess->start();
     m_debuggerProcess->waitForStarted();
-    auto dir = QStandardPaths::locate(QStandardPaths::GenericDataLocation,
-                                      QStringLiteral("kdevpythonsupport/debugger/"), QStandardPaths::LocateDirectory);
+
+    const auto dir = info.debuggerPath.section(QLatin1Char('/'), 0, -2, QString::SectionIncludeTrailingSep);
     InternalPdbCommand* path = new InternalPdbCommand(nullptr, nullptr,
         QStringLiteral("import sys; sys.path.append('") + dir + QStringLiteral("')\n"));
     InternalPdbCommand* cmd = new InternalPdbCommand(nullptr, nullptr, QStringLiteral("import __kdevpython_debugger_utils\n"));
@@ -277,7 +266,6 @@ void DebugSession::setState(DebuggerState state)
     m_state = state;
     if ( m_state == EndedState ) {
         raiseEvent(debugger_exited);
-        Q_EMIT finished();
     }
     else if ( m_state == ActiveState || m_state == StartingState || m_state == StoppingState ) {
         raiseEvent(debugger_busy);
